import multiprocessing
from multiprocessing import Pool
import subprocess
from drivescanner import file_index, ingest, rules
from typing import Tuple


def _scan_file(file_tuple: tuple, NER: bool = False) -> Tuple[str, dict]:
    """
    It takes a tuple of a file name and a dictionary of file information, reads the file, and then
    updates the dictionary with the results of the file scan
    
    Args:
      file_tuple (Tuple): Tuple
    
    Returns:
      A tuple of the dictkey and the file_dict
    """
    # input test before we go
    assert isinstance(file_tuple, tuple), "Input should be a tuple"

    dictkey = file_tuple[0]
    file_dict = file_tuple[1]

    content, errormsg = ingest.ingest_file(file_dict["filepath"])

    # unsync file from OneDrive (unclear if it works for other cloud providers)
    subprocess.run('attrib +U -P "' + file_dict["filepath"] + '"')

    if content is None:
        file_dict["content_processed"] = False
        file_dict["processing_error"] = errormsg
    else:
        file_dict["content_processed"] = True
        
        if NER == True:  
            file_dict["entities"] = rules.extract_NER(content, NER_select=["PERSON"])
            rules_dict = rules.search_text_file(content)
            file_dict.update(rules_dict)
        else:    
            rules_dict = rules.search_text_file(content)
            file_dict.update(rules_dict)

    return (dictkey, file_dict)


def scan_drive(list_files: list[str], NER: bool = False) -> dict:
    """
    It takes a list of files, creates a dictionary of those files, then uses the multiprocessing library
    to scan each file in parallel
    
    Args:
      list_files (list[str]): list[str]
    
    Returns:
      A dictionary of dictionaries.
    """
    resultdict = file_index.create_file_dict(list_files=list_files)

    pool = Pool(multiprocessing.cpu_count())
    results = pool.starmap(_scan_file, [(i, NER) for i in resultdict.items()] )

    return dict(results)



""" this module contains a function that will search text files for: bsn, iban, email, tel, zip, adress, cv types."""

import re
import spacy
import string
import phonenumbers
from phonenumbers import timezone
from langdetect import detect, LangDetectException

PRIVACY_TYPES = [
    "bsn",
    "iban",
    "email",
    "telnr",
    "zip",
    "address",
    "cv",
    "credentials",
    "creditcard",
    "passport",
]

CV_KEYWORDS = ["resume", "cv", "curriculum vitae", "personalia"]

CREDENTIALS_KEYWORDS = [
    "gebruiker",
    "gebruikersnaam",
    "account",
    "accountnaam",
    "user",
    "username",
    "usr",
    "wachtwoord",
    "ww",
    "password",
    "pwd",
    "pass",
    "credentials",
    "inloggegevens",
    "login",
    "pw",
]

LETTERS = {ord(d): str(i) for i, d in enumerate(string.digits + string.ascii_uppercase)}


def _remove_entities(txt: str, entity_list: list[str]) -> str:
    """
    It takes a string and removes all substrings given by a list of strings
    Args:
        txt (str): an input string / document from which you want to remove substrings
        found_list (list[str]): a list of substrings to remove.

    Returns:
        A string without the substrings given in the list
    """
    assert isinstance(txt, str), "txt should be a str"
    assert isinstance(entity_list, list), "found_list should be a list of strings"
    assert all(
        isinstance(elem, str) for elem in entity_list
    ), "found_list should be a list of strings"
    if list == []:
        return txt
    else:
        for l in entity_list:
            txt = re.sub(l, "", txt)
        return txt


def _extract_ssn(txt: str) -> list[str]:
    """
    It takes a string extracts all valid dutch social security numbers from it
        Args:
            txt (str): an input string from which we want to extract social security numbers from

        Returns:
            A list of valid found ssn
    """
    assert isinstance(txt, str), "txt should be a str"
    bsn_found_list = []
    bsn_findings = re.findall("([0-9]{9})", txt)

    bsn_lambda = (
        lambda i: 7
        < len(i)
        < 10
        * (
            (S := sum(a * b for a, b in zip([-1, *range(2, 10)], map(int, i[::-1]))))
            % 11
            < 1
        )
        * S
        > 0
    )

    for possible_bsn in bsn_findings:
        if bsn_lambda(possible_bsn):
            bsn_found_list.append(possible_bsn)

    return bsn_found_list


def _valid_iban(iban: str) -> bool:
    """
    It takes an iban number in string form and checks wether it is a valid iban
        Args:
            txt (str): an input string from which we want to extract iban numbers from

        Returns:
            A list of valid found iban numbers
    """
    assert isinstance(iban, str), "iban should be a str"
    if len(iban) < 18 or not iban.isalnum():
        return False
    else:
        iban = (iban[4:] + iban[:4]).translate(LETTERS)
        return int(iban) % 97 == 1


def _extract_iban(txt: str) -> list[str]:
    """
    It takes a string and extracts all valid iban numbers from it
        Args:
            txt (str): an input string from which we want to extract iban numbers from

        Returns:
            A list of valid found iban numbers
    """
    assert isinstance(txt, str), "txt should be a str"
    iban_found_list = []
    iban_findings = re.findall(
        "([a-zA-Z]{2}[0-9]{2}[a-zA-Z0-9]{4}[0-9]{7}([a-zA-Z0-9]?){0,16})", txt
    )
    iban_findings = list(set([item for t in iban_findings for item in t]))

    for possible_iban in iban_findings:
        if _valid_iban(possible_iban):
            iban_found_list.append(possible_iban)

    return iban_found_list


def _extract_email(txt: str) -> list[str]:
    """
    It takes a string and extracts all possible email addresses from it
        Args:
            txt (str): an input string from which we want to extract email addresses from

        Returns:
            A list of possible found email addresses
    """
    assert isinstance(txt, str), "txt should be a str"
    email_findings = re.findall(r"([\w\.-]+@[\w\.-]+\.\w+)", txt)
    return email_findings


def _extract_phonenumbers(txt: str) -> list[str]:
    """
    It takes a string and extracts all valid phone numbers from it
        Args:
            txt (str): an input string from which we want to extract phone numbers from

        Returns:
            A list of valid found phone numbers
    """
    assert isinstance(txt, str), "txt should be a str"
    phonenumbers_found_list = []

    # remove all characters that are not + or 0-9 characters
    txt = re.sub("[^0-9+]", "", txt)

    # extract all possible phonenumbers beginning with a country code like +31
    country_codes_found = re.findall(r"([\+][0-9]{10,14})", txt)

    # loop through all found country codes check for possible combinations if it is a valid number
    for nr in country_codes_found:
        possible_number = nr
        while len(possible_number) > 10:
            try:
                if phonenumbers.is_possible_number(phonenumbers.parse(possible_number)):
                    info = (
                        possible_number,
                        timezone.time_zones_for_number(
                            phonenumbers.parse(possible_number)
                        )[0],
                    )
                    phonenumbers_found_list.append(info)
                possible_number = possible_number[:-1]
            except:
                possible_number = possible_number[:-1]
                continue

    # extract all 06-x formatted mobile numbers
    mobile_found = re.findall("([0][6][0-9]{8})", txt)

    # check for all 06-x numbers if it is a valid number assuming the country code is +31
    for nr in mobile_found:
        possible_number = "+31" + nr[1:]
        if phonenumbers.is_possible_number(phonenumbers.parse(possible_number)):
            info = (
                nr,
                timezone.time_zones_for_number(phonenumbers.parse(possible_number))[0],
            )
            if ("+31" + info[0], info[1]) not in phonenumbers_found_list:
                phonenumbers_found_list.append(info)

    # remove our found phone numbers from which the region is unknown hence likely unvalid
    phonenumbers_found_list = [
        x for x in phonenumbers_found_list if x[1] != "Etc/Unknown"
    ]

    return phonenumbers_found_list


def _extract_zip(txt: str) -> list[str]:
    """
    It takes a string and extracts all valid dutch zipcodes from it
        Args:
            txt (str): an input string from which we want to extract zipcodes from

        Returns:
            A list of valid found zipcodes
    """
    assert isinstance(txt, str), "txt should be a str"

    # extract all zip codes using regex
    zip_found = re.findall(
        "([1-9][0-9][0-9][0-9][A-EGHJ-NPR-TVWXZ][A-EGHJ-NPR-TVWXZ])", txt
    )
    return zip_found


def _check_cv(txt: str) -> bool:
    """
    It takes a string and checks if there are CV keywords in it
        Args:
            txt (str): an input string from which we want to check if it contains CV keywords

        Returns:
            A boolean flag that indicate if the sting contains CV keywords
    """
    assert isinstance(txt, str), "txt should be a str"

    found_cv_words = []
    for keyword in CV_KEYWORDS:
        found_cv_words += re.findall("(" + keyword + ")", txt.lower())
    if len(found_cv_words) > 0:
        return True
    else:
        return False


def _check_credentials(txt: str) -> bool:
    """
    It takes a string and checks if there are credentials keywords in it
        Args:
            txt (str): an input string from which we want to check if it contains credential keywords

        Returns:
            A boolean flag that indicate if the sting contains credential keywords
    """
    assert isinstance(txt, str), "txt should be a str"

    found_credentials = []
    for keyword in CREDENTIALS_KEYWORDS:
        found_credentials += re.findall("(" + keyword + ")", txt.lower())
    if len(found_credentials) > 0:
        return True
    else:
        return False


def _valid_cc_number(card_number: str) -> bool:
    """
    It takes a credit card number in string and checks if the credit card number is valid using luhn's algorithm
        Args:
            card_number (str): an input that represents a credit card number

        Returns:
            A boolean flag that indicate if the sting contains credential keywords
    """
    assert isinstance(card_number, str), "card_number should be a str"

    # 1. Keep only numeric characters in string
    card_number = re.sub("[^0-9]", "", card_number)

    # 2. If not the right length return false
    if len(card_number) != 16:
        return False

    # 3. Change datatype to list[int]
    card_number = [int(num) for num in card_number]

    # 4. Remove the last digit:
    checkDigit = card_number.pop(-1)

    # 5. Reverse the remaining digits:
    card_number.reverse()

    # 6. Double digits at even indices
    card_number = [
        num * 2 if idx % 2 == 0 else num for idx, num in enumerate(card_number)
    ]

    # 7. Subtract 9 at even indices if digit is over 9
    # (or you can add the digits)
    card_number = [
        num - 9 if idx % 2 == 0 and num > 9 else num
        for idx, num in enumerate(card_number)
    ]

    # 8. Add the checkDigit back to the list:
    card_number.append(checkDigit)

    # 9. Sum all digits:
    checkSum = sum(card_number)

    # 10. If checkSum is divisible by 10, it is valid.
    return checkSum % 10 == 0


def _extract_cc_number(txt: str) -> list[str]:
    """
    It takes a string and checks if there are creditcard number available
        Args:
            txt (str): an input string from which we want to check if it contains a creditcard number

        Returns:
            A boolean flag that indicate if the sting contains a creditcard number
    """
    assert isinstance(txt, str), "txt should be a str"

    pattern = r"(?:\d{4}[ -]?){4}(?=[^\w]|$)"

    found_cc_number = re.findall(pattern, txt)
    found_cc_number = [x for x in found_cc_number if _valid_cc_number(x)]

    return found_cc_number


def _extract_passport(txt: str) -> list[str]:
    """
    It takes a string and extracts possible NL password numbers from the input. An NL passport number has the following layout
    pos 1 and 2: letters; pos 3-8: letters or digits; pos 9: digit. The letter 'O' is not used while the digit zero may be used.
        Args:
            txt (str): an input string from which we want to extract NL passport numbers

        Returns:
            A list of valid found passport numbers
    """
    assert isinstance(txt, str), "txt should be a str"

    passport_number = re.findall("[A-Z]{2}[A-Z0-9]{6}[0-9]{1}(?=[^A-Z0-9]|$)", txt)

    return passport_number


def search_text_file(text_file: str) -> dict:
    """
    It takes a string/text file to checks and validate the following privacy types:
    bsn, iban, email, tel, zip, cv and credentials
        Args:
            txt (str): an input string from which we want to extract privacy types

        Returns:
            A dictionary with lists of entities that are privacy types
    """
    assert isinstance(text_file, str), "text_file should be a str"

    output_dict = {}
    for type in PRIVACY_TYPES:
        output_dict[type] = []

    # extract iban from text
    iban_numbers = _extract_iban(text_file)
    output_dict["iban"] += iban_numbers
    text_file = _remove_entities(text_file, iban_numbers)

    # extract bsn numbers from text
    bsn_numbers = _extract_ssn(text_file)
    output_dict["bsn"] += bsn_numbers
    text_file = _remove_entities(text_file, bsn_numbers)

    # extract email from text
    emails = _extract_email(text_file)
    output_dict["email"] += emails
    text_file = _remove_entities(text_file, emails)

    # extract phone numbers
    phonenumbers = _extract_phonenumbers(text_file)
    output_dict["telnr"] += phonenumbers

    # extract zipcodes
    zipcodes = _extract_zip(text_file)
    output_dict["zip"] += zipcodes

    # check credentials and cv flags
    output_dict["credentials"] = _check_credentials(text_file)
    output_dict["cv"] = _check_cv(text_file)

    # check creditcard number
    creditcard_numbers = _extract_cc_number(text_file)
    output_dict["creditcard"] += creditcard_numbers

    # check for passport numbers
    passport_numbers = _extract_passport(text_file)
    output_dict["passport"] += passport_numbers

    return output_dict


def _load_spacy_model(model_name: str):
    try:
        ner = spacy.load(
            model_name,
            exclude=["tok2vec", "tagger", "parser", "attribute_ruler", "lemmatizer"],
        )
    except OSError:
        spacy.cli.download(model=model_name)
        package_path = spacy.util.get_package_path(model_name)
        spacy.cli.link(model_name, model_name, force=True, package_path=package_path)
        ner = spacy.load(
            model_name,
            exclude=["tok2vec", "tagger", "parser", "attribute_ruler", "lemmatizer"],
        )
    return ner


def _detect_language(input: str) -> str:
    try:
        lang = detect(input)
    except LangDetectException:
        lang = "unknown"
    return lang


def extract_NER(input: str, NER_select: list[str] = ["PERSON"]):
    """
    It takes a string as input and returns a list of choosen Named Entities as output

    Args:
        input (str): an input string / document from which you want to extract entities
        NER_select (list[str], optional): a list of possible entities to extract, defaults to ['PERSON'].

        PERSON:      People, including fictional.
        NORP:        Nationalities or religious or political groups.
        FAC:         Buildings, airports, highways, bridges, etc.
        ORG:         Companies, agencies, institutions, etc.
        GPE:         Countries, cities, states.
        LOC:         Non-GPE locations, mountain ranges, bodies of water.
        PRODUCT:     Objects, vehicles, foods, etc. (Not services.)
        EVENT:       Named hurricanes, battles, wars, sports events, etc.
        WORK_OF_ART: Titles of books, songs, etc.
        LAW:         Named documents made into laws.
        LANGUAGE:    Any named language.
        DATE:        Absolute or relative dates or periods.
        TIME:        Times smaller than a day.
        PERCENT:     Percentage, including ”%“.
        MONEY:       Monetary values, including unit.
        QUANTITY:    Measurements, as of weight or distance.
        ORDINAL:     “first”, “second”, etc.
        CARDINAL:    Numerals that do not fall under another type.

    Requirements:
        This function requires a Dutch Spacy model the be available
        Get it using: python -m spacy download nl_core_news_sm, en_core_web_sm, xx_ent_wiki_sm

    Returns:
        A list of strings with entities and entity types
    """
    # input test before we go
    assert isinstance(input, str), "Input should be a string"

    # Make sure we use the right lang-model
    lang = _detect_language(input)

    if lang == "nl":
        ner = _load_spacy_model("nl_core_news_sm")
    elif lang == "en":
        ner = _load_spacy_model("en_core_web_sm")
    else:  # multi-language fall-back
        ner = _load_spacy_model("xx_ent_wiki_sm")
    # use the Spacy lib to tokenize and parse the input text
    doc = ner(input)

    # create an empy list to store results
    entities = []

    # extract entity names and labels and add the entities to the dictionary
    for ent in (ent for ent in doc.ents if ent.label_ in NER_select):
        entity_text = ent.text
        entity_label = ent.label_
        entities.append([entity_text, entity_label])

    # return entities
    return entities



""" 
This file takes the output of the rules-script, transforms it to human readbale format and applies scoring on the severity of the found PII. 
"""

import pandas as pd
import warnings

def calculate_severity(
    resultdict: dict,
    scores: dict = {
        "bsn": 50,
        "iban": 20,
        "email": 1,
        "telnr": 10,
        "zip": 5,
        "address": 5,
        "cv": 30,
        "credentials": 5,
        "creditcard": 20,
        "passport": 50,
    },
) -> pd.DataFrame:
    """
    This is a function to calculate the severity of PII violations given the rules output. Each PII type is multiplied by a score. Score are summed at document level to reflect the severity of the violations found.

    Default scores are
    bsn: 50, discovery of Dutch Social Security Number
    iban: 20, international bank account number
    email: 1, email address
    telnr: 10, national or international phone number
    zip: 5, Dutch zipcode
    address: 5, address
    cv: 30, resume keywords detected
    credentials: 5, credential keywords detected
    passport: 50, Dutch password number

    Args:
        input (pd.DataFrame): it takes a Pandas dataframe as input containing boolean values in the columns for each PII type, the output of the function 'result_to_boolean_dataframe'
        scores (dict): the severtity scores for each PII type

    Returns:
        pd.DataFrame: when PII type has been detected a score is returned indicating the severity
    """
    resultdf = pd.DataFrame.from_dict(resultdict, orient="index")

    df_not_processed = resultdf[~resultdf.content_processed]
    resultdf = resultdf[resultdf.content_processed]

    # infer the datatypes of the columns where there are results
    resultdf = resultdf.infer_objects()

    if len(resultdf.index) > 0:
        for col in list(scores.keys()):
            if resultdf[col].dtype == "bool":
                resultdf[col + "_severity"] = resultdf[col] * scores[col]
            else:
                # assuming all others are lists
                resultdf[col + "_severity"] = resultdf[col].apply(len)
                resultdf[col + "_severity"].values[resultdf[col + "_severity"] > 1] = 1
                resultdf[col + "_severity"] = resultdf[col + "_severity"] * scores[col]

        score_cols = [col + "_severity" for col in scores.keys()]

        resultdf["total"] = resultdf[score_cols].sum(axis=1).astype(int)
        resultdf = resultdf.sort_values("total", ascending=False)
    else:
        warnings.warn("None of the files are processed, no risk score can be calculated")

    return resultdf, df_not_processed



# the xlrd version (1.2.0) used in Textractplus is unable to handle .xlsx files with the standard settings, below code fixes that problem
# source: https://stackoverflow.com/questions/64264563/attributeerror-elementtree-object-has-no-attribute-getiterator-when-trying
import xlrd
xlrd.xlsx.ensure_elementtree_imported(False, None)
xlrd.xlsx.Element_has_iter = True

from drivescanner.file_index import *
from drivescanner.scan import *
from drivescanner.evaluate import *


import textractplus as tp
from textractplus.exceptions import MissingFileError
from bs4 import UnicodeDammit, BeautifulSoup
import re
from typing import Tuple
from drivescanner.file_index import _replace_backslash
import PyPDF2


def _read_bytes(filepath: str) -> Tuple[bytearray, str]:
    """
    It reads the file and returns the content as a bytearray

    Args:
      filepath (str): the path to the file to be read

    Returns:
      A bytearray
    """
    try:
        content = tp.process(filepath)
        errormsg = None
    except ModuleNotFoundError:
        content = None
        errormsg = "Filetype cannot be processed"
    except MissingFileError:
        content = None
        errormsg = "File not found"
    except Exception as e:
        content = None
        errormsg = e
    return content, errormsg


def _decode_bytearray(
    bytestring: bytearray, try_encoding: list[str]
) -> Tuple[str, str]:
    """
    If the bytestring is not None, then try to decode it using the encoding derived
    from the bytestring or try to decode it using the encoding specified in the
    using some predefined encodings

    Args:
      bytestring (bytearray): the bytearray to decode
      try_encoding (list[str]): a list of encodings to try.

    Returns:
      the content of the bytestring as a decoded string
    """
    possible_encoding = UnicodeDammit(bytestring).original_encoding
    content = None
    if (possible_encoding is not None) & (possible_encoding not in try_encoding):
        try_encoding.insert(0, possible_encoding)
    for enc in try_encoding:
        if content is None:
            try:
                content = bytestring.decode(enc)
            except UnicodeDecodeError:
                pass
    if content is None:
        errormsg = "File encoding unknown"
    else:
        errormsg = None
    return content, errormsg


def _clean_text(text: str) -> Tuple[str, str]:
    """
    It removes HTML and XML characters, and replaces all new lines and white spaces like tabs with a
    space

    Args:
      text (str): the text to be cleaned

    Returns:
      A string with no new lines or tabs.
    """
    # remove HTML and XML characters
    text = BeautifulSoup(text).get_text()
    # replace all new lines and white spaces like tabs with a space
    text = re.sub(r"\s", " ", text)
    return text, None


def _pdf_to_string(pdf_file: str) -> Tuple[str, str]:
    """
    It opens the PDF file, reads the text from each page, and returns the text as a string

    Args:
      pdf_file (str): The path to the PDF file.

    Returns:
      A tuple of two strings, the content and/or the error message (if any) while processing the file.
    """
    try:
        with open(pdf_file, "rb") as pdf:
            reader = PyPDF2.PdfReader(pdf, strict=False)
            pdf_text = []

            for page in reader.pages:
                content = page.extract_text()
                pdf_text.append(content)

            content = " ".join(pdf_text)
            errormsg = None
    except FileNotFoundError:
        content = None
        errormsg = "File not found"
    except Exception as e:
        content = None
        errormsg = e
    return content, errormsg


def ingest_file(
    filepath: str, try_encoding: list[str] = ["utf-8", "iso-8859-1"]
) -> Tuple[str, str]:
    """
    It reads a file, tries to decode it, and then cleans it

    Args:
      filepath (str): str = the path to the file you want to ingest
      try_encoding (list[str]): list[str] = ["utf-8", "iso-8859-1"]

    Returns:
      A string
    """
    filepath = _replace_backslash(filepath)

    if filepath.lower().endswith(".pdf"):
        # use PyPDF2
        content, errormsg = _pdf_to_string(filepath)
    else:
        # use textractplus
        content, errormsg = _read_bytes(filepath=filepath)
        if (content is not None) and (errormsg is None):
            content, errormsg = _decode_bytearray(
                bytestring=content, try_encoding=try_encoding
            )

    if (content is not None) and (errormsg is None):
        content, errormsg = _clean_text(content)

    return content, errormsg



""" this module contains functions to scan all files in a directory """

import os
import pandas as pd
import hashlib
from collections import Counter
from pathlib import Path

def _replace_backslash(dir_path: str) -> str:
    """
    It takes a string as input and returns a string with all the backslashes replaced with forward
    slashes

    Args:
      dir_path (str): The path to the directory you want to convert.

    Returns:
      the string (path) with the backslash replaced with a forward slash.
    """
    return str(dir_path).replace(os.path.sep, "/")


def _create_hash(text: str) -> str:
    """
    It takes a string and returns the has value of that string using the MD5 hashing algorithm

    Args:
      text (str): The text to be hashed.

    Returns:
      A hash of the text.
    """
    return str(hashlib.md5(text.encode("utf-8")).hexdigest())


def _get_extension(filepath: str) -> str:
    """
    It takes a filepath as a string, splits it into the filename and extension, converts the extension
    to lowercase, and removes the leading period

    Args:
      filepath (str): The path to the file you want to get the extension of.

    Returns:
      The file extension of the filepath.
    """
    return os.path.splitext(filepath)[1].lower().replace(".", "", 1)


def _get_extension_list(list_files: list[str]) -> list[str]:
    """
    It takes a list of files, and returns a list with only the file extensions

    Args:
      list_files (list[str]): a list of filenames (which can include the filepath)

    Returns:
      A list of strings (file extensions)
    """
    assert isinstance(list_files, list), "Input should be a list of strings"
    assert all(
        isinstance(elem, str) for elem in list_files
    ), "Input should be a list of strings"
    assert len(list_files) > 0, "Input is an empty list"

    all_extensions = list(map(_get_extension, list_files))
    return all_extensions


def _get_filesize(filepath: str) -> int:
    """
    > This function returns the size of a file in bytes

    Args:
      filepath (str): The path to the file you want to get the size of.

    Returns:
      The size of the file in bytes.
    """
    try:
        size = os.path.getsize(filepath)
    except FileNotFoundError:
        size = None
    return size


def _get_filesize_list(list_files: list[str]) -> list[int]:
    """
    This function takes a list of file names and returns a list of file sizes in bytes

    Args:
      list_files (list[str]): list[str]

    Returns:
      A list of integers
    """
    assert isinstance(list_files, list), "Input should be a list of strings"
    assert all(
        isinstance(elem, str) for elem in list_files
    ), "Input should be a list of strings"
    assert len(list_files) > 0, "Input is an empty list"

    all_sizes = list(map(_get_filesize, list_files))
    return all_sizes


def list_all_files(dir_path: str) -> list[str]:
    """
    It takes a directory path as input and returns a list of file paths.

    Args:
      dir_path (str): The path to the directory containing the files.

    Returns:
      A list of strings.
    """
    dir_path = _replace_backslash(dir_path)

    if not os.path.exists(dir_path):
        raise ValueError(f"The directory doesn't exist.")

    if not os.path.isdir(dir_path):
        raise ValueError(f"The path doesn't point to a directory.")

    file_paths = []
    for dirname, _, files in os.walk(dir_path):
        file_paths += [os.path.join(dirname, entry) for entry in files]

    if len(file_paths) == 0:
        raise ValueError(f"Could not find any files in the directory.")

    return file_paths


def extension_stats(list_files: list[str]) -> pd.DataFrame:
    """
    It takes a list of files and returns a dataframe with the number of files per extension

    Args:
      list_files (list[str]): list[str]

    Returns:
      A dataframe with the extension and the count of files with that extension.
    """
    all_extensions = _get_extension_list(list_files=list_files)

    # count nr of files per extensions
    count_extensions = Counter([ext for ext in all_extensions])
    df_ext_stats = pd.DataFrame.from_dict(
        count_extensions, orient="index"
    ).reset_index()
    df_ext_stats.columns = ["extension", "count"]
    df_ext_stats = df_ext_stats.sort_values(by="count", ascending=False)
    return df_ext_stats


def select_files(
    list_files: list[str], include: list[str] = None, exclude: list[str] = None
) -> list[str]:
    """
    It takes a list of files, and returns a list of files that have extensions that are in the include
    list, but not in the exclude list

    Args:
      list_files (list[str]): list[str]
      include (list[str]): list[str] = None
      exclude (list[str]): list[str] = None

    Returns:
      list_files
    """

    if include is not None:
        assert isinstance(
            include, list
        ), "Include parameter should be a list of strings"
        assert all(
            isinstance(elem, str) for elem in include
        ), "Include parameter should be a list of strings"
        assert len(include) > 0, "Include parameter is an empty list"
        include = [elem.lower() for elem in include]

    if exclude is not None:
        assert isinstance(
            exclude, list
        ), "Exclude parameter should be a list of strings"
        assert all(
            isinstance(elem, str) for elem in exclude
        ), "Exclude parameter should be a list of strings"
        assert len(exclude) > 0, "Exclude parameter is an empty list"
        exclude = [elem.lower() for elem in exclude]

    if (include is not None) and (exclude is not None):
        list_files = [
            entry
            for entry, ext in zip(list_files, _get_extension_list(list_files))
            if (ext in include) & (ext not in exclude)
        ]
    elif (include is not None) and (exclude is None):
        list_files = [
            entry
            for entry, ext in zip(list_files, _get_extension_list(list_files))
            if ext in include
        ]
    elif (include is None) and (exclude is not None):
        list_files = [
            entry
            for entry, ext in zip(list_files, _get_extension_list(list_files))
            if ext not in exclude
        ]
    else:
        list_files

    return list_files


def create_file_dict(list_files: list[str]) -> dict:
    """
    It takes a list of file paths and returns a dictionary of file paths with their corresponding hash, extension and size

    Args:
      list_files (list[str]): list[str]

    Returns:
      A dictionary of dictionaries with information about the files.
    """
    all_extensions = _get_extension_list(list_files=list_files)
    all_sizes = _get_filesize_list(list_files=list_files)
    file_dict = {
        _create_hash(list_files[i]): {
            "filepath": list_files[i],
            "filename": Path(list_files[i]).name,
            "extension": all_extensions[i],
            "filesize": all_sizes[i],
        }
        for i in range(len(list_files))
    }
    return file_dict



from setuptools import setup, find_packages
from pathlib import Path

# read the contents of the README file
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="drivescanner",
    version="0.0.2",
    python_requires=">=3.9",
    description="Scan your filesystem to look for files that are a potential GDPR risk",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Wouter van Gils, Wim Verboom, Sem Frankenberg, Steven Goos, Rick Flamand, Jeanine Schoonemann",
    author_email="service@cmotions.nl",
    url="https://gitlab.com/cmotions/packages/drivescanner/",
    packages=find_packages(),
    install_requires=[
        "pandas",
        "textract-plus",
        "spacy",
        "phonenumbers",
        "langdetect",
        "bs4",
        "pypdf2"
    ],
    extras_require={
        "dev": [
            "black",
            "jupyterlab",
            "pytest>=6.2.4",
            "python-dotenv",
            "ipykernel",
            "twine"
        ],
    },
    # files to be shipped with the installation
    # after installation, these can be found with the functions in resources.py
    package_data={
        "drivescanner": [
            "data/*.csv",
            "data/*.txt",
            "notebooks/*tutorial*.ipynb",
        ]
    },
)
